/**********************************************************************
*
*     PROJECT: Drag & Drop sample with XbpBrowse Object (mouse & touch)
*  Programmer: J.A. Diego Kerejeta
*     Created: 08/27/2018
*
***********************************************************************/

#include 'Xbp.ch'
#include 'Appevent.ch'
#include 'gra.ch'

#pragma Library( "XppUi2.lib" )


*********************************************
PROCEDURE Sample2( oDlgOwner, lStandardDrag )
*********************************************
LOCAL nEvent, mp1, mp2, oDlg, oXbp, oBrowse, oBox, oListBox, nCont, oFocus, oDataArea
LOCAL aSize:= AppDesktop():currentSize()
LOCAL aPos := AClone( aSize )

aSize[ 1 ]:= Int( aSize[ 1 ] * 0.6 )
aSize[ 2 ]:= Int( aSize[ 2 ] * 0.7 )
aPos[ 1 ] := Int( ( aPos[ 1 ] - aSize[ 1 ] ) / 2 )
aPos[ 2 ] := Int( ( aPos[ 2 ] - aSize[ 2 ] ) / 2 )

oDlg := XbpDialog():new( AppDesktop(), oDlgOwner , aPos, aSize, { { XBP_PP_ORIGIN, XBP_ORIGIN_TOPLEFT }, ;
                         { XBP_PP_COMPOUNDNAME, '10.Arial' } }, .F. )
   oDlg:title                   := 'Drag & Drop with XbpBrowse (mouse & touch) ' + ;
                                   IIF( lStandardDrag, '[keeping the standard drag]', '[blocking the standard drag]' )
   oDlg:taskList                := .T.
   oDlg:drawingArea:clipChildren:= .T.
   oDlg:create()

OpenDbf()
oBrowse:= CreateBrowse( oDlg:drawingArea )

aSize:= oDlg:drawingArea:currentSize()
   aSize[ 1 ]:= Int( aSize[ 1 ] / 2 ) - 100
   aSize[ 2 ]:= aSize[ 2 ] - 100

oXbp:= XbpStatic():new()
   oXbp:type        := XBPSTATIC_TYPE_GROUPBOX
   oXbp:caption     := 'Drop Area'
   oXbp:clipChildren:= .T.
   oXbp:create( oDlg:drawingArea,, { aSize[ 1 ] + 140, 40 }, aSize )

aSize[ 1 ] -= 20
aSize[ 2 ] -= 30

oListBox:= XbpListBox():new()
   oListBox:create( oXbp,, { 10, 20 }, aSize )

FOR nCont:= 1 TO oBrowse:colCount
   oDataArea:= oBrowse:getColumn( nCont ):dataArea
      oDataArea:registraReceptor( oListBox, {|oEmisor, oReceptor| Drop( oEmisor, oReceptor ) } )
      oDataArea:lStandardDrag:= lStandardDrag
NEXT

oDlg:setModalState( XBP_DISP_APPMODAL )
oDlg:show()
oFocus:= SetAppFocus( oBrowse )

DO WHILE nEvent <> xbeP_Close
   nEvent := AppEvent( @mp1, @mp2, @oXbp )
   oXbp:handleEvent( nEvent, mp1, mp2 )
ENDDO
oDlg:setModalState( XBP_DISP_MODELESS )
oDlg:destroy()
SetAppFocus( oFocus )
KK->( DbCloseArea() )
RETURN 

************************************
PROCEDURE Drop( oEmisor, oReceptor )
************************************
LOCAL xData:= oEmisor:getDragData()
LOCAL cData, nCont

IF Valtype( xData ) # 'A'
   cData:= Alltrim( Var2Char( xData ) )
ELSEIF Len( xData ) = 0
   cData:= ''
ELSE
   cData:= Alltrim( Var2Char( xData[ nCont ] ) )
   FOR nCont:= 2 TO Len( xData )
      cData +=  ', ' + Alltrim( Var2Char( xData[ nCont ] ) )
   NEXT
ENDIF
oReceptor:addItem( cData )
oReceptor:setData( oReceptor:numItems() )
RETURN

********************************
FUNCTION CreateBrowse( oParent )
********************************
LOCAL aPP, oBrowse, oColumn
LOCAL aSize:= oParent:currentSize()

aSize[ 1 ]:= Int( aSize[ 1 ] / 2 )

oBrowse := XbpBrowse():new( oParent,, { 0, 0 }, aSize,, .F. )
   oBrowse:skipBlock     := {|n| DbSkipper(n) }
   oBrowse:goTopBlock    := {| | DbGoTop()    }
   oBrowse:goBottomBlock := {| | DbGoBottom() }
   oBrowse:phyPosBlock   := {| | Recno()      }
   oBrowse:posBlock      := {| | DbPosition()    }
   oBrowse:goPosBlock    := {|n| DbGoPosition(n) } 
   oBrowse:lastPosBlock  := {| | 100             }
   oBrowse:firstPosBlock := {| | 0               }
   oBrowse:cursorMode    := XBPBRW_CURSOR_CELL
   oBrowse:create()

aPP := { { XBP_PP_COL_HA_CAPTION, 'CAMPO_111' } }

oColumn := DragColumn():new( oBrowse,,,, aPP )
   oColumn:dataLink:= {|x| KK->CAMPO_111 }
   oBrowse:addColumn( oColumn )

aPP[ 1, 2 ] := 'CAMPO_222'

oColumn := DragColumn():new( oBrowse,,,, aPP )
   oColumn:dataLink:= {|x| KK->CAMPO_222 }
   oBrowse:addColumn( oColumn )
  
aPP[ 1, 2 ] := 'CAMPO_333'

oColumn := DragColumn():new( oBrowse,,,, aPP )
   oColumn:dataLink:= {|x| KK->CAMPO_333 }
   oBrowse:addColumn( oColumn )

aPP[ 1, 2 ] := 'CAMPO_444'

oColumn := DragColumn():new( oBrowse,,,, aPP )
   oColumn:dataLink:= {|x| KK->CAMPO_444 }
   oBrowse:addColumn( oColumn )

oBrowse:show()
RETURN oBrowse

*****************
PROCEDURE OpenDbf
*****************
LOCAL cPath:= StrTran( AppName( .T. ), AppName(), '' )
LOCAL aStruct, nCont

FIELD CAMPO_111, CAMPO_222, CAMPO_333, CAMPO_444

IF ! Fexists( cPath + 'KK.DBF' )
    aStruct   := { { 'CAMPO_111', 'C', 25, 0  }, ;
                   { 'CAMPO_222', 'N',  6, 0  }, ;
                   { 'CAMPO_333', 'C', 25, 0  }, ;
                   { 'CAMPO_444', 'C',  6, 0  } }

   DBcreate( cPath + 'KK', aStruct )

   USE ( cPath + 'KK' ) NEW EXCLUSIVE

   FOR nCont:= 1 TO 100
      KK->( DBappend() )
      KK->CAMPO_111:= CMONTH( DATE() + 31 * nCont )
      KK->CAMPO_222:= nCont + 1000
      KK->CAMPO_333:= CMONTH( DATE() + 31 * ( 100 - nCont ) )
      KK->CAMPO_444:= LTrim( Str( 100 - nCont + 1000 ) )
   NEXT
   KK->( DbCloseArea() )
END
USE ( cPath + 'KK' ) NEW SHARED
RETURN
